//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Numeric.cc	--- implementation of routines in Numeric namespace
//
//	DESCRIPTION:
//		Implementation of numerical auxiliary routines
//
//	CHANGE HISTORY:
//		UL, 2001-12-10: 0.1.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Dec 10 11:37:11 MET 2001
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /home/sasbuild/repositories/sasdev/sas/packages/utils/src/Numeric.cc,v 1.1 2001/12/10 17:21:29 ulammers Exp $
///

#include <cmath>

#include "Numeric.h"

using namespace std;

namespace Numeric {

double gammaln(double xx)
{
	static double coef[] = { 76.18009172947146, -86.50532032941677,
						     24.01409824083091, -1.231739572450155,
							 .1208650973866179e-2, -.5395239384953e-5 };

	double x, y;
	y = x = xx;
	double tmp = x+5.5;
	tmp -= (x+.5)*log(tmp);
	double ser = 1.000000000190015;
	for (unsigned j=0; j<=5; ++j)
		ser += coef[j]/++y;

	return -tmp+log(2.5066282746310005*ser/x);
} // gammaln()


double factorial(unsigned short n)
{
	const unsigned short NMAX = 32;
	static unsigned _nact = 4;
	static double _facnum[NMAX+1] = { 1., 1., 2., 6., 24. };

	if (n > NMAX)
		return gammaln(n+1.);	//	n! = gammaln(n+1)

	while (_nact < n) {
		unsigned short j = _nact++;
		_facnum[_nact] = _facnum[j]*_nact;
	}

	return _facnum[n];
}

}
