

#include <ctime>


#include "Tracer.h"
#include "StopWatch.h"


class Clock
{
	public :

		virtual ~Clock() {}
		virtual void stop() = 0;
		virtual bool stopped() const = 0;
		virtual void start() = 0;
		virtual void split() = 0;
		virtual double splitElapsed() const = 0;
		virtual StopWatch::Type type() const = 0;
		virtual double elapsed() const = 0;
		virtual void reset() = 0;

};


class CPUClock : public virtual Clock
{
	public :
		CPUClock( bool autoStart ) {
			PROC
			if( autoStart )
			{
				_start = clock();
				_stopped = false;
			}
			else
			{
				_start = std::clock();
				_stop = _start;
				_stopped = true;
			}// if
			ENDPROC
		}
		~CPUClock() { PROC ENDPROC }
		void stop() {
			PROC
			if( !_stopped ) {
				_stop = std::clock();
				_stopped = true;
			} // if
			ENDPROC
		}
		bool stopped() const { PROC RETURN _stopped; }
		void start() {
			PROC
			if( _stopped )
			{
				_start = std::clock() - _stop + _start;
				_stopped = false;
			} // if
			ENDPROC
		}
		void split() { PROC ENDPROC }
		double splitElapsed() const { PROC RETURN 0.0; }
		StopWatch::Type type() const { PROC RETURN StopWatch::CPU; }
		double elapsed() const {
			PROC
			double	etime;
			if( _stopped ) etime = std::difftime( _stop, _start );
			else etime = difftime( std::clock(), _start );
			RETURN etime/ CLOCKS_PER_SEC;
		}
		void reset() { PROC _start = std::clock(); ENDPROC }
	private :
		clock_t	_start;
		clock_t	_stop;
		bool	_stopped;
};


class CalenderClock : public virtual Clock
{
	public :
		CalenderClock( bool autoStart ) {
			PROC
			if( autoStart )
			{
				_start = std::time(0);
				_stopped = false;
			}
			else
			{
				_start = std::time(0);
				_stop = _start;
				_stopped = true;
			}// if
			ENDPROC
		}
		~CalenderClock() { PROC ENDPROC }
		void stop() {
			PROC
			if( !_stopped ) {
				_stop = std::time(0);
				_stopped = true;
			} // if
			ENDPROC
		}
		bool stopped() const { PROC RETURN _stopped; }
		void start() {
			PROC
			if( _stopped )
			{
				_start = std::time(0) - _stop + _start;
				_stopped = false;
			} // if
			ENDPROC
		}
		void split() { PROC ENDPROC }
		double splitElapsed() const { PROC RETURN 0.0; }
		StopWatch::Type type() const { PROC RETURN StopWatch::Calender; }
		double elapsed() const {
			PROC
			double etime;
			if( _stopped ) etime = std::difftime( _stop, _start );
			else etime = std::difftime( std::time(0), _start );
			RETURN etime;
		}
		void reset() { PROC _start = std::time(0); ENDPROC }
	private :
		time_t	_start;
		time_t	_stop;
		bool	_stopped;
};


StopWatch::StopWatch( Type type /*= CPU*/, bool autoStart /*= true*/ )
{
	PROC
	switch( type )
	{
		case StopWatch::CPU :
			_clock = new CPUClock( autoStart );
			break;
		case StopWatch::Calender :
			_clock = new CalenderClock( autoStart );
			break;
		default :
			std::cerr << "Unrecognised clock type: " << int(type) << std::endl;
			std::exit( 1 );
			break;
	} // if
	ENDPROC
}


StopWatch::~StopWatch()
{
	PROC
	delete _clock;
	ENDPROC
}


void
StopWatch::stop()
{
	PROC
	_clock -> stop ();
	ENDPROC
}


bool
StopWatch::stopped() const
{
	PROC
	RETURN _clock -> stopped();
}


void
StopWatch::start()
{
	PROC
	_clock -> start();
	ENDPROC
}


void
StopWatch::split()
{
	PROC
	_clock -> split();
	ENDPROC
}


double
StopWatch::splitElapsed() const
{
	PROC
	RETURN _clock -> splitElapsed();
}


StopWatch::Type
StopWatch::type() const
{
	PROC
	RETURN _clock -> type();
}


double
StopWatch::elapsed() const
{
	PROC
	RETURN _clock -> elapsed();
}


void
StopWatch::reset()
{
	PROC
	_clock -> reset();
	ENDPROC
}

